unit uDiscoverDifferencesUI;
{$INCLUDE 'Plugin.inc'}

interface

uses
  SysUtils, Windows, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uPluginConsts, uMessages,
  uOptions;

type
  TDiscoverDifferencesDialog = class(TOkCancelDialog)
  private
    fLeftPanel: TFarPanel;
    fRightPanel: TFarPanel;
    fOptions: TOptions;
    fActualOptions: TOptions;
    fLabelTitle: TFarDialog.TLabel;
    fCheckProcessSubfolders: TFarDialog.TCheckbox;
    fCheckUseMaxScanDepth: TFarDialog.TCheckbox;
    fEditMaxScanDepth: TFarDialog.TEdit;
    fCheckProcessSelected: TFarDialog.TCheckbox;
    fSeparator1: TFarDialog.TSeparator;
    fLabelCompareBox: TFarDialog.TLabel;
    fCheckCompareTime: TFarDialog.TCheckbox;
    fCheckCompareLowPrecision: TFarDialog.TCheckbox;
    fCheckIgnoreTimeZone: TFarDialog.TCheckbox;
    fCheckCompareSize: TFarDialog.TCheckbox;
    fCheckCompareFileNameCase: TFarDialog.TCheckbox;
    fCheckCompareContents: TFarDialog.TCheckbox;
    fCheckCompareContentsIgnore: TFarDialog.TCheckbox;
    fRadioIgnoreNewLines: TFarDialog.TRadioButton;
    fRadioIgnoreWhiteSpace: TFarDialog.TRadioButton;
    fSeparator2: TFarDialog.TSeparator;
    fCheckSelectOnBothPanels: TFarDialog.TCheckbox;
  protected
    procedure SetSize(Width, Height: Integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    procedure OptionsToDialog;
    procedure DialogToOptions(Options: TOptions; ForSaving: boolean);
    procedure UpdateControls(ActiveCheckbox: TFarDialog.TCheckbox; ActiveCheckboxFinalState: TFarDialog.TCheckbox.TCheckState);
    procedure CheckboxClicked(Sender: TFarDialog.TCheckbox; FinalState: TFarDialog.TCheckbox.TCheckState; var AllowChange: boolean);
    property LabelTitle: TFarDialog.TLabel read fLabelTitle;
    property CheckProcessSubfolders: TFarDialog.TCheckbox read fCheckProcessSubfolders;
    property CheckUseMaxScanDepth: TFarDialog.TCheckbox read fCheckUseMaxScanDepth;
    property EditMaxScanDepth: TFarDialog.TEdit read fEditMaxScanDepth;
    property CheckProcessSelected: TFarDialog.TCheckbox read fCheckProcessSelected;
    property Separator1: TFarDialog.TSeparator read fSeparator1;
    property LabelCompareBox: TFarDialog.TLabel read fLabelCompareBox;
    property CheckCompareTime: TFarDialog.TCheckbox read fCheckCompareTime;
    property CheckCompareLowPrecision: TFarDialog.TCheckbox read fCheckCompareLowPrecision;
    property CheckIgnoreTimeZone: TFarDialog.TCheckbox read fCheckIgnoreTimeZone;
    property CheckCompareSize: TFarDialog.TCheckbox read fCheckCompareSize;
    property CheckCompareFileNameCase: TFarDialog.TCheckbox read fCheckCompareFileNameCase;
    property CheckCompareContents: TFarDialog.TCheckbox read fCheckCompareContents;
    property CheckCompareContentsIgnore: TFarDialog.TCheckbox read fCheckCompareContentsIgnore;
    property RadioIgnoreNewLines: TFarDialog.TRadioButton read fRadioIgnoreNewLines;
    property RadioIgnoreWhiteSpace: TFarDialog.TRadioButton read fRadioIgnoreWhiteSpace;
    property Separator2: TFarDialog.TSeparator read fSeparator2;
    property CheckSelectOnBothPanels: TFarDialog.TCheckbox read fCheckSelectOnBothPanels;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; override;
    function Execute: boolean;
    property LeftPanel: TFarPanel read fLeftPanel write fLeftPanel;
    property RightPanel: TFarPanel read fRightPanel write fRightPanel;
    property Options: TOptions read fOptions write fOptions;
    property ActualOptions: TOptions read fActualOptions;
  end;

implementation

{ TDirSyncMainDialog }

procedure TDiscoverDifferencesDialog.CheckboxClicked(Sender: TFarDialog.TCheckbox; FinalState: TFarDialog.TCheckbox.TCheckState; var AllowChange: boolean);
begin
  UpdateControls(Sender, FinalState);
end;

procedure TDiscoverDifferencesDialog.Clear;
begin
  inherited;
  fLabelTitle := nil;
  fCheckProcessSubfolders := nil;
  fCheckUseMaxScanDepth := nil;
  fEditMaxScanDepth := nil;
  fCheckProcessSelected := nil;
  fSeparator1 := nil;
  fLabelCompareBox := nil;
  fCheckCompareTime := nil;
  fCheckCompareLowPrecision := nil;
  fCheckIgnoreTimeZone := nil;
  fCheckCompareSize := nil;
  fCheckCompareFileNameCase := nil;
  fCheckCompareContents := nil;
  fCheckCompareContentsIgnore := nil;
  fRadioIgnoreNewLines := nil;
  fRadioIgnoreWhiteSpace := nil;
  fSeparator2 := nil;
  fCheckSelectOnBothPanels := nil;
end;

procedure TDiscoverDifferencesDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
begin
  inherited;
  Width := 66;
  Height := 23;
end;

constructor TDiscoverDifferencesDialog.Create;
begin
  inherited Create(DIALOG_MAIN_GUID);
  fActualOptions := TOptions.Create;
  fLabelTitle := AddLabel(0, 0, 0, GetMsg(MProcessBox));
  fCheckProcessSubfolders := AddCheckbox(0, 0, GetMsg(MProcessSubfolders), False);
  fCheckProcessSubfolders.OnClick := CheckboxClicked;
  fCheckUseMaxScanDepth := AddCheckbox(0, 0, GetMsg(MUseMaxScanDepth), False);
  fCheckUseMaxScanDepth.OnClick := CheckboxClicked;
  fEditMaxScanDepth := AddEdit(fCheckUseMaxScanDepth.Left + Length(fCheckUseMaxScanDepth.Text) + 5, fCheckUseMaxScanDepth.Top, 5, '99');
  fEditMaxScanDepth.Mask := '99999';
  fEditMaxScanDepth.MaxLength := Length(fEditMaxScanDepth.Mask);
  fEditMaxScanDepth.Width := fEditMaxScanDepth.MaxLength + 1;
  fCheckProcessSelected := AddCheckbox(0, 0, GetMsg(MProcessSelected), False);
  fSeparator1 := AddSeparator(0);
  fLabelCompareBox := AddLabel(0, 0, 0, GetMsg(MCompareBox));
  fCheckCompareFileNameCase := AddCheckbox(0, 0, GetMsg(MCompareFileNameCase), False);
  fCheckCompareTime := AddCheckbox(0, 0, GetMsg(MCompareTime), False);
  fCheckCompareTime.OnClick := CheckboxClicked;
  fCheckCompareLowPrecision := AddCheckbox(0, 0, GetMsg(MCompareLowPrecision), False);
  fCheckIgnoreTimeZone := AddCheckbox(0, 0, GetMsg(MCompareIgnoreTimeZone), False);
  fCheckCompareSize := AddCheckbox(0, 0, GetMsg(MCompareSize), False);
  fCheckCompareContents := AddCheckbox(0, 0, GetMsg(MCompareContents), False);
  fCheckCompareContents.OnClick := CheckboxClicked;
  fCheckCompareContentsIgnore := AddCheckbox(0, 0, GetMsg(MCompareContentsIgnore), False);
  fCheckCompareContentsIgnore.OnClick := CheckboxClicked;
  fRadioIgnoreNewLines := AddRadioButton(0, 0, GetMsg(MCompareIgnoreNewLines), False);
  fRadioIgnoreWhiteSpace := AddRadioButton(0, 0, GetMsg(MCompareIgnoreWhitespace), False);
  fSeparator2 := AddSeparator(0);
  fCheckSelectOnBothPanels := AddCheckbox(0, 0, GetMsg(MSelectOnBothPanels), False);
end;

destructor TDiscoverDifferencesDialog.Destroy;
begin
  FreeAndNil(fActualOptions);
  inherited;
end;

procedure TDiscoverDifferencesDialog.DialogToOptions(Options: TOptions; ForSaving: boolean);
begin
  Options.ProcessSubfolders := CheckProcessSubfolders.Checked and (ForSaving or (CheckProcessSubfolders.Enabled));
  Options.UseMaxScanDepth := CheckUseMaxScanDepth.Checked and (ForSaving or (Options.ProcessSubfolders and CheckUseMaxScanDepth.Enabled));
  Options.MaxScanDepth := StrToIntDef(EditMaxScanDepth.Text, Options.MaxScanDepth);
  Options.ProcessSelected := CheckProcessSelected.Checked and (ForSaving or (CheckProcessSelected.Enabled));
  Options.CompareTime := CheckCompareTime.Checked and (ForSaving or (CheckCompareTime.Enabled));
  Options.LowPrecisionTime := CheckCompareLowPrecision.Checked and (ForSaving or (Options.CompareTime and CheckCompareLowPrecision.Enabled));
  Options.IgnorePossibleTimeZoneDifferences := CheckIgnoreTimeZone.Checked and (ForSaving or (Options.CompareTime and CheckIgnoreTimeZone.Enabled));
  Options.CompareSize := CheckCompareSize.Checked and (ForSaving or (CheckCompareSize.Enabled));
  Options.CompareFileNameCase := CheckCompareFileNameCase.Checked and (ForSaving or (CheckCompareFileNameCase.Enabled));
  Options.CompareContents := CheckCompareContents.Checked and (ForSaving or (CheckCompareContents.Enabled));
  Options.CompareContentsIgnore := CheckCompareContentsIgnore.Checked and (ForSaving or (Options.CompareContents and CheckCompareContentsIgnore.Enabled));
  Options.IgnoreNewLines := RadioIgnoreNewLines.Checked and (ForSaving or (Options.CompareContentsIgnore and RadioIgnoreNewLines.Enabled));
  Options.IgnoreWhitespace := RadioIgnoreWhiteSpace.Checked and (ForSaving or (Options.CompareContentsIgnore and RadioIgnoreWhiteSpace.Enabled));
  Options.SelectDifferencesOnBothPanels := CheckSelectOnBothPanels.Checked and (ForSaving or (CheckSelectOnBothPanels.Enabled));
end;

function TDiscoverDifferencesDialog.Execute: boolean;
begin
  Result := False;
  ActualOptions.Assign(Options);
  OptionsToDialog;
  if inherited Execute then
  begin
    DialogToOptions(Options, True);
    DialogToOptions(ActualOptions, False);
    Result := True;
  end;
end;

procedure TDiscoverDifferencesDialog.OptionsToDialog;
begin
  CheckProcessSubfolders.Checked := Options.ProcessSubfolders;
  CheckUseMaxScanDepth.Checked := Options.UseMaxScanDepth;
  EditMaxScanDepth.Text := IntToStr(Options.MaxScanDepth);
  CheckProcessSelected.Checked := LeftPanel.HasSelectedItems or RightPanel.HasSelectedItems;
  CheckCompareTime.Checked := Options.CompareTime;
  CheckCompareLowPrecision.Checked := Options.LowPrecisionTime;
  CheckIgnoreTimeZone.Checked := Options.IgnorePossibleTimeZoneDifferences;
  CheckCompareSize.Checked := Options.CompareSize;
  CheckCompareFileNameCase.Checked := Options.CompareFileNameCase;
  CheckCompareContents.Checked := Options.CompareContents;
  CheckCompareContentsIgnore.Checked := Options.CompareContentsIgnore;
  RadioIgnoreNewLines.Checked := Options.IgnoreNewLines or (not Options.IgnoreWhitespace);
  RadioIgnoreWhiteSpace.Checked := Options.IgnoreWhitespace and (not Options.IgnoreNewLines);
  CheckSelectOnBothPanels.Checked := Options.SelectDifferencesOnBothPanels;
  UpdateControls(nil, csUnchecked);
end;

procedure TDiscoverDifferencesDialog.SetSize(Width, Height: Integer);
begin
  inherited;
  AutoPos(LabelTitle);
  AutoPos(CheckProcessSubfolders);
  AutoPos(CheckUseMaxScanDepth, 4);
  AutoPosRight(EditMaxScanDepth, CheckUseMaxScanDepth, 2, 0, EditMaxScanDepth.MaxLength + 1);
  AutoPos(CheckProcessSelected);
  AutoPos(Separator1);
  AutoPos(LabelCompareBox);
  AutoPos(CheckCompareFileNameCase);
  AutoPos(CheckCompareTime);
  AutoPos(CheckCompareLowPrecision, 4);
  AutoPos(CheckIgnoreTimeZone, 4);
  AutoPos(CheckCompareSize);
  AutoPos(CheckCompareContents);
  AutoPos(CheckCompareContentsIgnore, 4);
  AutoPos(RadioIgnoreNewLines, 8);
  AutoPos(RadioIgnoreWhiteSpace, 8);
  AutoPos(Separator2);
  AutoPos(CheckSelectOnBothPanels);
end;

procedure TDiscoverDifferencesDialog.UpdateControls(ActiveCheckbox: TFarDialog.TCheckbox; ActiveCheckboxFinalState: TFarDialog.TCheckbox.TCheckState);

  function IsChecked(Checkbox: TFarDialog.TCheckbox): boolean;
  begin
    if ActiveCheckbox = Checkbox then
      Result := ActiveCheckboxFinalState = csChecked
    else
      Result := Checkbox.Checked;
  end;

begin
  CheckProcessSubfolders.Enabled := (not LeftPanel.IsPluginPanel) and (not RightPanel.IsPluginPanel);
  CheckUseMaxScanDepth.Enabled := CheckProcessSubfolders.Enabled and IsChecked(CheckProcessSubfolders);
  EditMaxScanDepth.Enabled := CheckUseMaxScanDepth.Enabled and IsChecked(CheckUseMaxScanDepth);
  CheckProcessSelected.Enabled := LeftPanel.HasSelectedItems or RightPanel.HasSelectedItems;
  CheckCompareLowPrecision.Enabled := CheckCompareTime.Enabled and IsChecked(CheckCompareTime);
  CheckIgnoreTimeZone.Enabled := CheckCompareTime.Enabled and IsChecked(CheckCompareTime);
  CheckCompareContents.Enabled := (not LeftPanel.IsPluginPanel) and (not RightPanel.IsPluginPanel);
  CheckCompareContentsIgnore.Enabled := CheckCompareContents.Enabled and IsChecked(CheckCompareContents);
  RadioIgnoreNewLines.Enabled := CheckCompareContentsIgnore.Enabled and IsChecked(CheckCompareContentsIgnore);
  RadioIgnoreWhiteSpace.Enabled := CheckCompareContentsIgnore.Enabled and IsChecked(CheckCompareContentsIgnore);
end;

end.
